# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
import base64
from typing import Any, Union, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    NOT_GIVEN,
    Omit,
    Headers,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
)
from ._utils import (
    is_given,
    get_async_library,
)
from ._models import FinalRequestOptions
from ._version import __version__
from .resources import (
    plans,
    events,
    meters,
    accounts,
    counters,
    pricings,
    products,
    webhooks,
    bill_jobs,
    contracts,
    currencies,
    bill_config,
    commitments,
    plan_groups,
    aggregations,
    account_plans,
    custom_fields,
    debit_reasons,
    authentication,
    credit_reasons,
    plan_templates,
    resource_groups,
    counter_pricings,
    plan_group_links,
    external_mappings,
    transaction_types,
    counter_adjustments,
    organization_config,
    permission_policies,
    compound_aggregations,
    integration_configurations,
    notification_configurations,
    scheduled_event_configurations,
)
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import M3terError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)
from .resources.bills import bills
from .resources.usage import usage
from .resources.users import users
from .resources.balances import balances
from .resources.data_exports import data_exports

__all__ = ["Timeout", "Transport", "ProxiesTypes", "RequestOptions", "M3ter", "AsyncM3ter", "Client", "AsyncClient"]

from .types import AuthenticationGetBearerTokenResponse


class M3ter(SyncAPIClient):
    authentication: authentication.AuthenticationResource
    accounts: accounts.AccountsResource
    account_plans: account_plans.AccountPlansResource
    aggregations: aggregations.AggregationsResource
    balances: balances.BalancesResource
    bills: bills.BillsResource
    bill_config: bill_config.BillConfigResource
    commitments: commitments.CommitmentsResource
    bill_jobs: bill_jobs.BillJobsResource
    compound_aggregations: compound_aggregations.CompoundAggregationsResource
    contracts: contracts.ContractsResource
    counters: counters.CountersResource
    counter_adjustments: counter_adjustments.CounterAdjustmentsResource
    counter_pricings: counter_pricings.CounterPricingsResource
    credit_reasons: credit_reasons.CreditReasonsResource
    currencies: currencies.CurrenciesResource
    custom_fields: custom_fields.CustomFieldsResource
    data_exports: data_exports.DataExportsResource
    debit_reasons: debit_reasons.DebitReasonsResource
    events: events.EventsResource
    external_mappings: external_mappings.ExternalMappingsResource
    integration_configurations: integration_configurations.IntegrationConfigurationsResource
    meters: meters.MetersResource
    notification_configurations: notification_configurations.NotificationConfigurationsResource
    organization_config: organization_config.OrganizationConfigResource
    permission_policies: permission_policies.PermissionPoliciesResource
    plans: plans.PlansResource
    plan_groups: plan_groups.PlanGroupsResource
    plan_group_links: plan_group_links.PlanGroupLinksResource
    plan_templates: plan_templates.PlanTemplatesResource
    pricings: pricings.PricingsResource
    products: products.ProductsResource
    resource_groups: resource_groups.ResourceGroupsResource
    scheduled_event_configurations: scheduled_event_configurations.ScheduledEventConfigurationsResource
    transaction_types: transaction_types.TransactionTypesResource
    usage: usage.UsageResource
    users: users.UsersResource
    webhooks: webhooks.WebhooksResource
    with_raw_response: M3terWithRawResponse
    with_streaming_response: M3terWithStreamedResponse

    # client options
    api_key: str
    api_secret: str
    token: str | None
    org_id: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        api_secret: str | None = None,
        token: str | None = None,
        org_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous M3ter client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `M3TER_API_KEY`
        - `api_secret` from `M3TER_API_SECRET`
        - `token` from `M3TER_API_TOKEN`
        - `org_id` from `M3TER_ORG_ID`
        """
        if api_key is None:
            api_key = os.environ.get("M3TER_API_KEY")
        if api_key is None:
            raise M3terError(
                "The api_key client option must be set either by passing api_key to the client or by setting the M3TER_API_KEY environment variable"
            )
        self.api_key = api_key

        if api_secret is None:
            api_secret = os.environ.get("M3TER_API_SECRET")
        if api_secret is None:
            raise M3terError(
                "The api_secret client option must be set either by passing api_secret to the client or by setting the M3TER_API_SECRET environment variable"
            )
        self.api_secret = api_secret

        if token is None:
            token = os.environ.get("M3TER_API_TOKEN")
        self.token = token

        if org_id is None:
            org_id = os.environ.get("M3TER_ORG_ID")
        if org_id is None:
            raise M3terError(
                "The org_id client option must be set either by passing org_id to the client or by setting the M3TER_ORG_ID environment variable"
            )
        self.org_id = org_id

        if base_url is None:
            base_url = os.environ.get("M3TER_BASE_URL")
        if base_url is None:
            base_url = f"https://api.m3ter.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.authentication = authentication.AuthenticationResource(self)
        self.accounts = accounts.AccountsResource(self)
        self.account_plans = account_plans.AccountPlansResource(self)
        self.aggregations = aggregations.AggregationsResource(self)
        self.balances = balances.BalancesResource(self)
        self.bills = bills.BillsResource(self)
        self.bill_config = bill_config.BillConfigResource(self)
        self.commitments = commitments.CommitmentsResource(self)
        self.bill_jobs = bill_jobs.BillJobsResource(self)
        self.compound_aggregations = compound_aggregations.CompoundAggregationsResource(self)
        self.contracts = contracts.ContractsResource(self)
        self.counters = counters.CountersResource(self)
        self.counter_adjustments = counter_adjustments.CounterAdjustmentsResource(self)
        self.counter_pricings = counter_pricings.CounterPricingsResource(self)
        self.credit_reasons = credit_reasons.CreditReasonsResource(self)
        self.currencies = currencies.CurrenciesResource(self)
        self.custom_fields = custom_fields.CustomFieldsResource(self)
        self.data_exports = data_exports.DataExportsResource(self)
        self.debit_reasons = debit_reasons.DebitReasonsResource(self)
        self.events = events.EventsResource(self)
        self.external_mappings = external_mappings.ExternalMappingsResource(self)
        self.integration_configurations = integration_configurations.IntegrationConfigurationsResource(self)
        self.meters = meters.MetersResource(self)
        self.notification_configurations = notification_configurations.NotificationConfigurationsResource(self)
        self.organization_config = organization_config.OrganizationConfigResource(self)
        self.permission_policies = permission_policies.PermissionPoliciesResource(self)
        self.plans = plans.PlansResource(self)
        self.plan_groups = plan_groups.PlanGroupsResource(self)
        self.plan_group_links = plan_group_links.PlanGroupLinksResource(self)
        self.plan_templates = plan_templates.PlanTemplatesResource(self)
        self.pricings = pricings.PricingsResource(self)
        self.products = products.ProductsResource(self)
        self.resource_groups = resource_groups.ResourceGroupsResource(self)
        self.scheduled_event_configurations = scheduled_event_configurations.ScheduledEventConfigurationsResource(self)
        self.transaction_types = transaction_types.TransactionTypesResource(self)
        self.usage = usage.UsageResource(self)
        self.users = users.UsersResource(self)
        self.webhooks = webhooks.WebhooksResource(self)
        self.with_raw_response = M3terWithRawResponse(self)
        self.with_streaming_response = M3terWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        token = self.token
        if token is None:
            return {}
        return {"Authorization": f"Bearer {token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    @override
    def _validate_headers(self, headers: Headers, custom_headers: Headers) -> None:
        if custom_headers.get("Authorization"):
            return
        if self.token and headers.get("Authorization"):
            return
        if isinstance(custom_headers.get("Authorization"), Omit):
            return

        raise TypeError(
            '"Could not resolve authentication method. Expected the token to be set. Or for the `Authorization` headers to be explicitly omitted"'
        )

    @override
    def _prepare_options(
        self,
        options: FinalRequestOptions,  # noqa: ARG002
    ) -> FinalRequestOptions:
        if not options.url.endswith("/oauth/token"):
            if not self.token:
                auth: str = base64.b64encode(f"{self.api_key}:{self.api_secret}".encode("utf8")).decode("utf8")
                token: AuthenticationGetBearerTokenResponse = self.authentication.get_bearer_token(
                    grant_type="client_credentials", extra_headers={"Authorization": f"Basic {auth}"}
                )
                self.token = token.access_token
        return options

    def copy(
        self,
        *,
        api_key: str | None = None,
        api_secret: str | None = None,
        token: str | None = None,
        org_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            api_secret=api_secret or self.api_secret,
            token=token or self.token,
            org_id=org_id or self.org_id,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    def _get_org_id_path_param(self) -> str:
        return self.org_id

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncM3ter(AsyncAPIClient):
    authentication: authentication.AsyncAuthenticationResource
    accounts: accounts.AsyncAccountsResource
    account_plans: account_plans.AsyncAccountPlansResource
    aggregations: aggregations.AsyncAggregationsResource
    balances: balances.AsyncBalancesResource
    bills: bills.AsyncBillsResource
    bill_config: bill_config.AsyncBillConfigResource
    commitments: commitments.AsyncCommitmentsResource
    bill_jobs: bill_jobs.AsyncBillJobsResource
    compound_aggregations: compound_aggregations.AsyncCompoundAggregationsResource
    contracts: contracts.AsyncContractsResource
    counters: counters.AsyncCountersResource
    counter_adjustments: counter_adjustments.AsyncCounterAdjustmentsResource
    counter_pricings: counter_pricings.AsyncCounterPricingsResource
    credit_reasons: credit_reasons.AsyncCreditReasonsResource
    currencies: currencies.AsyncCurrenciesResource
    custom_fields: custom_fields.AsyncCustomFieldsResource
    data_exports: data_exports.AsyncDataExportsResource
    debit_reasons: debit_reasons.AsyncDebitReasonsResource
    events: events.AsyncEventsResource
    external_mappings: external_mappings.AsyncExternalMappingsResource
    integration_configurations: integration_configurations.AsyncIntegrationConfigurationsResource
    meters: meters.AsyncMetersResource
    notification_configurations: notification_configurations.AsyncNotificationConfigurationsResource
    organization_config: organization_config.AsyncOrganizationConfigResource
    permission_policies: permission_policies.AsyncPermissionPoliciesResource
    plans: plans.AsyncPlansResource
    plan_groups: plan_groups.AsyncPlanGroupsResource
    plan_group_links: plan_group_links.AsyncPlanGroupLinksResource
    plan_templates: plan_templates.AsyncPlanTemplatesResource
    pricings: pricings.AsyncPricingsResource
    products: products.AsyncProductsResource
    resource_groups: resource_groups.AsyncResourceGroupsResource
    scheduled_event_configurations: scheduled_event_configurations.AsyncScheduledEventConfigurationsResource
    transaction_types: transaction_types.AsyncTransactionTypesResource
    usage: usage.AsyncUsageResource
    users: users.AsyncUsersResource
    webhooks: webhooks.AsyncWebhooksResource
    with_raw_response: AsyncM3terWithRawResponse
    with_streaming_response: AsyncM3terWithStreamedResponse

    # client options
    api_key: str
    api_secret: str
    token: str | None
    org_id: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        api_secret: str | None = None,
        token: str | None = None,
        org_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncM3ter client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `M3TER_API_KEY`
        - `api_secret` from `M3TER_API_SECRET`
        - `token` from `M3TER_API_TOKEN`
        - `org_id` from `M3TER_ORG_ID`
        """
        if api_key is None:
            api_key = os.environ.get("M3TER_API_KEY")
        if api_key is None:
            raise M3terError(
                "The api_key client option must be set either by passing api_key to the client or by setting the M3TER_API_KEY environment variable"
            )
        self.api_key = api_key

        if api_secret is None:
            api_secret = os.environ.get("M3TER_API_SECRET")
        if api_secret is None:
            raise M3terError(
                "The api_secret client option must be set either by passing api_secret to the client or by setting the M3TER_API_SECRET environment variable"
            )
        self.api_secret = api_secret

        if token is None:
            token = os.environ.get("M3TER_API_TOKEN")
        self.token = token

        if org_id is None:
            org_id = os.environ.get("M3TER_ORG_ID")
        if org_id is None:
            raise M3terError(
                "The org_id client option must be set either by passing org_id to the client or by setting the M3TER_ORG_ID environment variable"
            )
        self.org_id = org_id

        if base_url is None:
            base_url = os.environ.get("M3TER_BASE_URL")
        if base_url is None:
            base_url = f"https://api.m3ter.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.authentication = authentication.AsyncAuthenticationResource(self)
        self.accounts = accounts.AsyncAccountsResource(self)
        self.account_plans = account_plans.AsyncAccountPlansResource(self)
        self.aggregations = aggregations.AsyncAggregationsResource(self)
        self.balances = balances.AsyncBalancesResource(self)
        self.bills = bills.AsyncBillsResource(self)
        self.bill_config = bill_config.AsyncBillConfigResource(self)
        self.commitments = commitments.AsyncCommitmentsResource(self)
        self.bill_jobs = bill_jobs.AsyncBillJobsResource(self)
        self.compound_aggregations = compound_aggregations.AsyncCompoundAggregationsResource(self)
        self.contracts = contracts.AsyncContractsResource(self)
        self.counters = counters.AsyncCountersResource(self)
        self.counter_adjustments = counter_adjustments.AsyncCounterAdjustmentsResource(self)
        self.counter_pricings = counter_pricings.AsyncCounterPricingsResource(self)
        self.credit_reasons = credit_reasons.AsyncCreditReasonsResource(self)
        self.currencies = currencies.AsyncCurrenciesResource(self)
        self.custom_fields = custom_fields.AsyncCustomFieldsResource(self)
        self.data_exports = data_exports.AsyncDataExportsResource(self)
        self.debit_reasons = debit_reasons.AsyncDebitReasonsResource(self)
        self.events = events.AsyncEventsResource(self)
        self.external_mappings = external_mappings.AsyncExternalMappingsResource(self)
        self.integration_configurations = integration_configurations.AsyncIntegrationConfigurationsResource(self)
        self.meters = meters.AsyncMetersResource(self)
        self.notification_configurations = notification_configurations.AsyncNotificationConfigurationsResource(self)
        self.organization_config = organization_config.AsyncOrganizationConfigResource(self)
        self.permission_policies = permission_policies.AsyncPermissionPoliciesResource(self)
        self.plans = plans.AsyncPlansResource(self)
        self.plan_groups = plan_groups.AsyncPlanGroupsResource(self)
        self.plan_group_links = plan_group_links.AsyncPlanGroupLinksResource(self)
        self.plan_templates = plan_templates.AsyncPlanTemplatesResource(self)
        self.pricings = pricings.AsyncPricingsResource(self)
        self.products = products.AsyncProductsResource(self)
        self.resource_groups = resource_groups.AsyncResourceGroupsResource(self)
        self.scheduled_event_configurations = scheduled_event_configurations.AsyncScheduledEventConfigurationsResource(
            self
        )
        self.transaction_types = transaction_types.AsyncTransactionTypesResource(self)
        self.usage = usage.AsyncUsageResource(self)
        self.users = users.AsyncUsersResource(self)
        self.webhooks = webhooks.AsyncWebhooksResource(self)
        self.with_raw_response = AsyncM3terWithRawResponse(self)
        self.with_streaming_response = AsyncM3terWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        token = self.token
        if token is None:
            return {}
        return {"Authorization": f"Bearer {token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    @override
    def _validate_headers(self, headers: Headers, custom_headers: Headers) -> None:
        if self.token and headers.get("Authorization"):
            return
        if isinstance(custom_headers.get("Authorization"), Omit):
            return

        raise TypeError(
            '"Could not resolve authentication method. Expected the token to be set. Or for the `Authorization` headers to be explicitly omitted"'
        )

    def copy(
        self,
        *,
        api_key: str | None = None,
        api_secret: str | None = None,
        token: str | None = None,
        org_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            api_secret=api_secret or self.api_secret,
            token=token or self.token,
            org_id=org_id or self.org_id,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    def _get_org_id_path_param(self) -> str:
        return self.org_id

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class M3terWithRawResponse:
    def __init__(self, client: M3ter) -> None:
        self.authentication = authentication.AuthenticationResourceWithRawResponse(client.authentication)
        self.accounts = accounts.AccountsResourceWithRawResponse(client.accounts)
        self.account_plans = account_plans.AccountPlansResourceWithRawResponse(client.account_plans)
        self.aggregations = aggregations.AggregationsResourceWithRawResponse(client.aggregations)
        self.balances = balances.BalancesResourceWithRawResponse(client.balances)
        self.bills = bills.BillsResourceWithRawResponse(client.bills)
        self.bill_config = bill_config.BillConfigResourceWithRawResponse(client.bill_config)
        self.commitments = commitments.CommitmentsResourceWithRawResponse(client.commitments)
        self.bill_jobs = bill_jobs.BillJobsResourceWithRawResponse(client.bill_jobs)
        self.compound_aggregations = compound_aggregations.CompoundAggregationsResourceWithRawResponse(
            client.compound_aggregations
        )
        self.contracts = contracts.ContractsResourceWithRawResponse(client.contracts)
        self.counters = counters.CountersResourceWithRawResponse(client.counters)
        self.counter_adjustments = counter_adjustments.CounterAdjustmentsResourceWithRawResponse(
            client.counter_adjustments
        )
        self.counter_pricings = counter_pricings.CounterPricingsResourceWithRawResponse(client.counter_pricings)
        self.credit_reasons = credit_reasons.CreditReasonsResourceWithRawResponse(client.credit_reasons)
        self.currencies = currencies.CurrenciesResourceWithRawResponse(client.currencies)
        self.custom_fields = custom_fields.CustomFieldsResourceWithRawResponse(client.custom_fields)
        self.data_exports = data_exports.DataExportsResourceWithRawResponse(client.data_exports)
        self.debit_reasons = debit_reasons.DebitReasonsResourceWithRawResponse(client.debit_reasons)
        self.events = events.EventsResourceWithRawResponse(client.events)
        self.external_mappings = external_mappings.ExternalMappingsResourceWithRawResponse(client.external_mappings)
        self.integration_configurations = integration_configurations.IntegrationConfigurationsResourceWithRawResponse(
            client.integration_configurations
        )
        self.meters = meters.MetersResourceWithRawResponse(client.meters)
        self.notification_configurations = (
            notification_configurations.NotificationConfigurationsResourceWithRawResponse(
                client.notification_configurations
            )
        )
        self.organization_config = organization_config.OrganizationConfigResourceWithRawResponse(
            client.organization_config
        )
        self.permission_policies = permission_policies.PermissionPoliciesResourceWithRawResponse(
            client.permission_policies
        )
        self.plans = plans.PlansResourceWithRawResponse(client.plans)
        self.plan_groups = plan_groups.PlanGroupsResourceWithRawResponse(client.plan_groups)
        self.plan_group_links = plan_group_links.PlanGroupLinksResourceWithRawResponse(client.plan_group_links)
        self.plan_templates = plan_templates.PlanTemplatesResourceWithRawResponse(client.plan_templates)
        self.pricings = pricings.PricingsResourceWithRawResponse(client.pricings)
        self.products = products.ProductsResourceWithRawResponse(client.products)
        self.resource_groups = resource_groups.ResourceGroupsResourceWithRawResponse(client.resource_groups)
        self.scheduled_event_configurations = (
            scheduled_event_configurations.ScheduledEventConfigurationsResourceWithRawResponse(
                client.scheduled_event_configurations
            )
        )
        self.transaction_types = transaction_types.TransactionTypesResourceWithRawResponse(client.transaction_types)
        self.usage = usage.UsageResourceWithRawResponse(client.usage)
        self.users = users.UsersResourceWithRawResponse(client.users)
        self.webhooks = webhooks.WebhooksResourceWithRawResponse(client.webhooks)


class AsyncM3terWithRawResponse:
    def __init__(self, client: AsyncM3ter) -> None:
        self.authentication = authentication.AsyncAuthenticationResourceWithRawResponse(client.authentication)
        self.accounts = accounts.AsyncAccountsResourceWithRawResponse(client.accounts)
        self.account_plans = account_plans.AsyncAccountPlansResourceWithRawResponse(client.account_plans)
        self.aggregations = aggregations.AsyncAggregationsResourceWithRawResponse(client.aggregations)
        self.balances = balances.AsyncBalancesResourceWithRawResponse(client.balances)
        self.bills = bills.AsyncBillsResourceWithRawResponse(client.bills)
        self.bill_config = bill_config.AsyncBillConfigResourceWithRawResponse(client.bill_config)
        self.commitments = commitments.AsyncCommitmentsResourceWithRawResponse(client.commitments)
        self.bill_jobs = bill_jobs.AsyncBillJobsResourceWithRawResponse(client.bill_jobs)
        self.compound_aggregations = compound_aggregations.AsyncCompoundAggregationsResourceWithRawResponse(
            client.compound_aggregations
        )
        self.contracts = contracts.AsyncContractsResourceWithRawResponse(client.contracts)
        self.counters = counters.AsyncCountersResourceWithRawResponse(client.counters)
        self.counter_adjustments = counter_adjustments.AsyncCounterAdjustmentsResourceWithRawResponse(
            client.counter_adjustments
        )
        self.counter_pricings = counter_pricings.AsyncCounterPricingsResourceWithRawResponse(client.counter_pricings)
        self.credit_reasons = credit_reasons.AsyncCreditReasonsResourceWithRawResponse(client.credit_reasons)
        self.currencies = currencies.AsyncCurrenciesResourceWithRawResponse(client.currencies)
        self.custom_fields = custom_fields.AsyncCustomFieldsResourceWithRawResponse(client.custom_fields)
        self.data_exports = data_exports.AsyncDataExportsResourceWithRawResponse(client.data_exports)
        self.debit_reasons = debit_reasons.AsyncDebitReasonsResourceWithRawResponse(client.debit_reasons)
        self.events = events.AsyncEventsResourceWithRawResponse(client.events)
        self.external_mappings = external_mappings.AsyncExternalMappingsResourceWithRawResponse(
            client.external_mappings
        )
        self.integration_configurations = (
            integration_configurations.AsyncIntegrationConfigurationsResourceWithRawResponse(
                client.integration_configurations
            )
        )
        self.meters = meters.AsyncMetersResourceWithRawResponse(client.meters)
        self.notification_configurations = (
            notification_configurations.AsyncNotificationConfigurationsResourceWithRawResponse(
                client.notification_configurations
            )
        )
        self.organization_config = organization_config.AsyncOrganizationConfigResourceWithRawResponse(
            client.organization_config
        )
        self.permission_policies = permission_policies.AsyncPermissionPoliciesResourceWithRawResponse(
            client.permission_policies
        )
        self.plans = plans.AsyncPlansResourceWithRawResponse(client.plans)
        self.plan_groups = plan_groups.AsyncPlanGroupsResourceWithRawResponse(client.plan_groups)
        self.plan_group_links = plan_group_links.AsyncPlanGroupLinksResourceWithRawResponse(client.plan_group_links)
        self.plan_templates = plan_templates.AsyncPlanTemplatesResourceWithRawResponse(client.plan_templates)
        self.pricings = pricings.AsyncPricingsResourceWithRawResponse(client.pricings)
        self.products = products.AsyncProductsResourceWithRawResponse(client.products)
        self.resource_groups = resource_groups.AsyncResourceGroupsResourceWithRawResponse(client.resource_groups)
        self.scheduled_event_configurations = (
            scheduled_event_configurations.AsyncScheduledEventConfigurationsResourceWithRawResponse(
                client.scheduled_event_configurations
            )
        )
        self.transaction_types = transaction_types.AsyncTransactionTypesResourceWithRawResponse(
            client.transaction_types
        )
        self.usage = usage.AsyncUsageResourceWithRawResponse(client.usage)
        self.users = users.AsyncUsersResourceWithRawResponse(client.users)
        self.webhooks = webhooks.AsyncWebhooksResourceWithRawResponse(client.webhooks)


class M3terWithStreamedResponse:
    def __init__(self, client: M3ter) -> None:
        self.authentication = authentication.AuthenticationResourceWithStreamingResponse(client.authentication)
        self.accounts = accounts.AccountsResourceWithStreamingResponse(client.accounts)
        self.account_plans = account_plans.AccountPlansResourceWithStreamingResponse(client.account_plans)
        self.aggregations = aggregations.AggregationsResourceWithStreamingResponse(client.aggregations)
        self.balances = balances.BalancesResourceWithStreamingResponse(client.balances)
        self.bills = bills.BillsResourceWithStreamingResponse(client.bills)
        self.bill_config = bill_config.BillConfigResourceWithStreamingResponse(client.bill_config)
        self.commitments = commitments.CommitmentsResourceWithStreamingResponse(client.commitments)
        self.bill_jobs = bill_jobs.BillJobsResourceWithStreamingResponse(client.bill_jobs)
        self.compound_aggregations = compound_aggregations.CompoundAggregationsResourceWithStreamingResponse(
            client.compound_aggregations
        )
        self.contracts = contracts.ContractsResourceWithStreamingResponse(client.contracts)
        self.counters = counters.CountersResourceWithStreamingResponse(client.counters)
        self.counter_adjustments = counter_adjustments.CounterAdjustmentsResourceWithStreamingResponse(
            client.counter_adjustments
        )
        self.counter_pricings = counter_pricings.CounterPricingsResourceWithStreamingResponse(client.counter_pricings)
        self.credit_reasons = credit_reasons.CreditReasonsResourceWithStreamingResponse(client.credit_reasons)
        self.currencies = currencies.CurrenciesResourceWithStreamingResponse(client.currencies)
        self.custom_fields = custom_fields.CustomFieldsResourceWithStreamingResponse(client.custom_fields)
        self.data_exports = data_exports.DataExportsResourceWithStreamingResponse(client.data_exports)
        self.debit_reasons = debit_reasons.DebitReasonsResourceWithStreamingResponse(client.debit_reasons)
        self.events = events.EventsResourceWithStreamingResponse(client.events)
        self.external_mappings = external_mappings.ExternalMappingsResourceWithStreamingResponse(
            client.external_mappings
        )
        self.integration_configurations = (
            integration_configurations.IntegrationConfigurationsResourceWithStreamingResponse(
                client.integration_configurations
            )
        )
        self.meters = meters.MetersResourceWithStreamingResponse(client.meters)
        self.notification_configurations = (
            notification_configurations.NotificationConfigurationsResourceWithStreamingResponse(
                client.notification_configurations
            )
        )
        self.organization_config = organization_config.OrganizationConfigResourceWithStreamingResponse(
            client.organization_config
        )
        self.permission_policies = permission_policies.PermissionPoliciesResourceWithStreamingResponse(
            client.permission_policies
        )
        self.plans = plans.PlansResourceWithStreamingResponse(client.plans)
        self.plan_groups = plan_groups.PlanGroupsResourceWithStreamingResponse(client.plan_groups)
        self.plan_group_links = plan_group_links.PlanGroupLinksResourceWithStreamingResponse(client.plan_group_links)
        self.plan_templates = plan_templates.PlanTemplatesResourceWithStreamingResponse(client.plan_templates)
        self.pricings = pricings.PricingsResourceWithStreamingResponse(client.pricings)
        self.products = products.ProductsResourceWithStreamingResponse(client.products)
        self.resource_groups = resource_groups.ResourceGroupsResourceWithStreamingResponse(client.resource_groups)
        self.scheduled_event_configurations = (
            scheduled_event_configurations.ScheduledEventConfigurationsResourceWithStreamingResponse(
                client.scheduled_event_configurations
            )
        )
        self.transaction_types = transaction_types.TransactionTypesResourceWithStreamingResponse(
            client.transaction_types
        )
        self.usage = usage.UsageResourceWithStreamingResponse(client.usage)
        self.users = users.UsersResourceWithStreamingResponse(client.users)
        self.webhooks = webhooks.WebhooksResourceWithStreamingResponse(client.webhooks)


class AsyncM3terWithStreamedResponse:
    def __init__(self, client: AsyncM3ter) -> None:
        self.authentication = authentication.AsyncAuthenticationResourceWithStreamingResponse(client.authentication)
        self.accounts = accounts.AsyncAccountsResourceWithStreamingResponse(client.accounts)
        self.account_plans = account_plans.AsyncAccountPlansResourceWithStreamingResponse(client.account_plans)
        self.aggregations = aggregations.AsyncAggregationsResourceWithStreamingResponse(client.aggregations)
        self.balances = balances.AsyncBalancesResourceWithStreamingResponse(client.balances)
        self.bills = bills.AsyncBillsResourceWithStreamingResponse(client.bills)
        self.bill_config = bill_config.AsyncBillConfigResourceWithStreamingResponse(client.bill_config)
        self.commitments = commitments.AsyncCommitmentsResourceWithStreamingResponse(client.commitments)
        self.bill_jobs = bill_jobs.AsyncBillJobsResourceWithStreamingResponse(client.bill_jobs)
        self.compound_aggregations = compound_aggregations.AsyncCompoundAggregationsResourceWithStreamingResponse(
            client.compound_aggregations
        )
        self.contracts = contracts.AsyncContractsResourceWithStreamingResponse(client.contracts)
        self.counters = counters.AsyncCountersResourceWithStreamingResponse(client.counters)
        self.counter_adjustments = counter_adjustments.AsyncCounterAdjustmentsResourceWithStreamingResponse(
            client.counter_adjustments
        )
        self.counter_pricings = counter_pricings.AsyncCounterPricingsResourceWithStreamingResponse(
            client.counter_pricings
        )
        self.credit_reasons = credit_reasons.AsyncCreditReasonsResourceWithStreamingResponse(client.credit_reasons)
        self.currencies = currencies.AsyncCurrenciesResourceWithStreamingResponse(client.currencies)
        self.custom_fields = custom_fields.AsyncCustomFieldsResourceWithStreamingResponse(client.custom_fields)
        self.data_exports = data_exports.AsyncDataExportsResourceWithStreamingResponse(client.data_exports)
        self.debit_reasons = debit_reasons.AsyncDebitReasonsResourceWithStreamingResponse(client.debit_reasons)
        self.events = events.AsyncEventsResourceWithStreamingResponse(client.events)
        self.external_mappings = external_mappings.AsyncExternalMappingsResourceWithStreamingResponse(
            client.external_mappings
        )
        self.integration_configurations = (
            integration_configurations.AsyncIntegrationConfigurationsResourceWithStreamingResponse(
                client.integration_configurations
            )
        )
        self.meters = meters.AsyncMetersResourceWithStreamingResponse(client.meters)
        self.notification_configurations = (
            notification_configurations.AsyncNotificationConfigurationsResourceWithStreamingResponse(
                client.notification_configurations
            )
        )
        self.organization_config = organization_config.AsyncOrganizationConfigResourceWithStreamingResponse(
            client.organization_config
        )
        self.permission_policies = permission_policies.AsyncPermissionPoliciesResourceWithStreamingResponse(
            client.permission_policies
        )
        self.plans = plans.AsyncPlansResourceWithStreamingResponse(client.plans)
        self.plan_groups = plan_groups.AsyncPlanGroupsResourceWithStreamingResponse(client.plan_groups)
        self.plan_group_links = plan_group_links.AsyncPlanGroupLinksResourceWithStreamingResponse(
            client.plan_group_links
        )
        self.plan_templates = plan_templates.AsyncPlanTemplatesResourceWithStreamingResponse(client.plan_templates)
        self.pricings = pricings.AsyncPricingsResourceWithStreamingResponse(client.pricings)
        self.products = products.AsyncProductsResourceWithStreamingResponse(client.products)
        self.resource_groups = resource_groups.AsyncResourceGroupsResourceWithStreamingResponse(client.resource_groups)
        self.scheduled_event_configurations = (
            scheduled_event_configurations.AsyncScheduledEventConfigurationsResourceWithStreamingResponse(
                client.scheduled_event_configurations
            )
        )
        self.transaction_types = transaction_types.AsyncTransactionTypesResourceWithStreamingResponse(
            client.transaction_types
        )
        self.usage = usage.AsyncUsageResourceWithStreamingResponse(client.usage)
        self.users = users.AsyncUsersResourceWithStreamingResponse(client.users)
        self.webhooks = webhooks.AsyncWebhooksResourceWithStreamingResponse(client.webhooks)


Client = M3ter

AsyncClient = AsyncM3ter



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

import httpx

from ..types import (
    plan_group_link_list_params,
    plan_group_link_create_params,
    plan_group_link_update_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.plan_group_link_response import PlanGroupLinkResponse

__all__ = ["PlanGroupLinksResource", "AsyncPlanGroupLinksResource"]


class PlanGroupLinksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PlanGroupLinksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return PlanGroupLinksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PlanGroupLinksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return PlanGroupLinksResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        plan_group_id: str,
        plan_id: str,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanGroupLinkResponse:
        """
        Create a new PlanGroupLink.

        Args:
          plan_group_id

          plan_id

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/plangrouplinks",
            body=maybe_transform(
                {
                    "plan_group_id": plan_group_id,
                    "plan_id": plan_id,
                    "version": version,
                },
                plan_group_link_create_params.PlanGroupLinkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanGroupLinkResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanGroupLinkResponse:
        """
        Retrieve a PlanGroupLink for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/plangrouplinks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanGroupLinkResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        plan_group_id: str,
        plan_id: str,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanGroupLinkResponse:
        """
        Update PlanGroupLink for the given UUID.

        Args:
          plan_group_id

          plan_id

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/plangrouplinks/{id}",
            body=maybe_transform(
                {
                    "plan_group_id": plan_group_id,
                    "plan_id": plan_id,
                    "version": version,
                },
                plan_group_link_update_params.PlanGroupLinkUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanGroupLinkResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        plan: str | NotGiven = NOT_GIVEN,
        plan_group: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[PlanGroupLinkResponse]:
        """
        Retrieve a list of PlanGroupLink entities

        Args:
          ids: list of IDs to retrieve

          next_token: nextToken for multi page retrievals

          page_size: Number of PlanGroupLinks to retrieve per page

          plan: UUID of the Plan to retrieve PlanGroupLinks for

          plan_group: UUID of the PlanGroup to retrieve PlanGroupLinks for

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/plangrouplinks",
            page=SyncCursor[PlanGroupLinkResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "plan": plan,
                        "plan_group": plan_group,
                    },
                    plan_group_link_list_params.PlanGroupLinkListParams,
                ),
            ),
            model=PlanGroupLinkResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanGroupLinkResponse:
        """
        Delete a PlanGroupLink for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/plangrouplinks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanGroupLinkResponse,
        )


class AsyncPlanGroupLinksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPlanGroupLinksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPlanGroupLinksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPlanGroupLinksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncPlanGroupLinksResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        plan_group_id: str,
        plan_id: str,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanGroupLinkResponse:
        """
        Create a new PlanGroupLink.

        Args:
          plan_group_id

          plan_id

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/plangrouplinks",
            body=await async_maybe_transform(
                {
                    "plan_group_id": plan_group_id,
                    "plan_id": plan_id,
                    "version": version,
                },
                plan_group_link_create_params.PlanGroupLinkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanGroupLinkResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanGroupLinkResponse:
        """
        Retrieve a PlanGroupLink for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/plangrouplinks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanGroupLinkResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        plan_group_id: str,
        plan_id: str,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanGroupLinkResponse:
        """
        Update PlanGroupLink for the given UUID.

        Args:
          plan_group_id

          plan_id

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/plangrouplinks/{id}",
            body=await async_maybe_transform(
                {
                    "plan_group_id": plan_group_id,
                    "plan_id": plan_id,
                    "version": version,
                },
                plan_group_link_update_params.PlanGroupLinkUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanGroupLinkResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        plan: str | NotGiven = NOT_GIVEN,
        plan_group: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[PlanGroupLinkResponse, AsyncCursor[PlanGroupLinkResponse]]:
        """
        Retrieve a list of PlanGroupLink entities

        Args:
          ids: list of IDs to retrieve

          next_token: nextToken for multi page retrievals

          page_size: Number of PlanGroupLinks to retrieve per page

          plan: UUID of the Plan to retrieve PlanGroupLinks for

          plan_group: UUID of the PlanGroup to retrieve PlanGroupLinks for

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/plangrouplinks",
            page=AsyncCursor[PlanGroupLinkResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "plan": plan,
                        "plan_group": plan_group,
                    },
                    plan_group_link_list_params.PlanGroupLinkListParams,
                ),
            ),
            model=PlanGroupLinkResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanGroupLinkResponse:
        """
        Delete a PlanGroupLink for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/plangrouplinks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanGroupLinkResponse,
        )


class PlanGroupLinksResourceWithRawResponse:
    def __init__(self, plan_group_links: PlanGroupLinksResource) -> None:
        self._plan_group_links = plan_group_links

        self.create = to_raw_response_wrapper(
            plan_group_links.create,
        )
        self.retrieve = to_raw_response_wrapper(
            plan_group_links.retrieve,
        )
        self.update = to_raw_response_wrapper(
            plan_group_links.update,
        )
        self.list = to_raw_response_wrapper(
            plan_group_links.list,
        )
        self.delete = to_raw_response_wrapper(
            plan_group_links.delete,
        )


class AsyncPlanGroupLinksResourceWithRawResponse:
    def __init__(self, plan_group_links: AsyncPlanGroupLinksResource) -> None:
        self._plan_group_links = plan_group_links

        self.create = async_to_raw_response_wrapper(
            plan_group_links.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            plan_group_links.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            plan_group_links.update,
        )
        self.list = async_to_raw_response_wrapper(
            plan_group_links.list,
        )
        self.delete = async_to_raw_response_wrapper(
            plan_group_links.delete,
        )


class PlanGroupLinksResourceWithStreamingResponse:
    def __init__(self, plan_group_links: PlanGroupLinksResource) -> None:
        self._plan_group_links = plan_group_links

        self.create = to_streamed_response_wrapper(
            plan_group_links.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            plan_group_links.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            plan_group_links.update,
        )
        self.list = to_streamed_response_wrapper(
            plan_group_links.list,
        )
        self.delete = to_streamed_response_wrapper(
            plan_group_links.delete,
        )


class AsyncPlanGroupLinksResourceWithStreamingResponse:
    def __init__(self, plan_group_links: AsyncPlanGroupLinksResource) -> None:
        self._plan_group_links = plan_group_links

        self.create = async_to_streamed_response_wrapper(
            plan_group_links.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            plan_group_links.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            plan_group_links.update,
        )
        self.list = async_to_streamed_response_wrapper(
            plan_group_links.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            plan_group_links.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .transaction_response import TransactionResponse as TransactionResponse
from .transaction_list_params import TransactionListParams as TransactionListParams
from .transaction_create_params import TransactionCreateParams as TransactionCreateParams
from .transaction_summary_response import TransactionSummaryResponse as TransactionSummaryResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["BillApproveParams"]


class BillApproveParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    bill_ids: Required[Annotated[List[str], PropertyInfo(alias="billIds")]]
    """Use to specify a collection of Bills by their IDs for batch approval"""

    account_ids: Annotated[str, PropertyInfo(alias="accountIds")]
    """List of Account IDs to filter Bills.

    This allows you to approve Bills for specific Accounts within the Organization.
    """

    external_invoice_date_end: Annotated[str, PropertyInfo(alias="externalInvoiceDateEnd")]
    """End date for filtering Bills by external invoice date.

    Includes Bills with dates earlier than this date.
    """

    external_invoice_date_start: Annotated[str, PropertyInfo(alias="externalInvoiceDateStart")]
    """Start date for filtering Bills by external invoice date.

    Includes Bills with dates equal to or later than this date.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, overload

import httpx

from .jobs import (
    JobsResource,
    AsyncJobsResource,
    JobsResourceWithRawResponse,
    AsyncJobsResourceWithRawResponse,
    JobsResourceWithStreamingResponse,
    AsyncJobsResourceWithStreamingResponse,
)
from ...types import data_export_create_adhoc_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    required_args,
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from .schedules import (
    SchedulesResource,
    AsyncSchedulesResource,
    SchedulesResourceWithRawResponse,
    AsyncSchedulesResourceWithRawResponse,
    SchedulesResourceWithStreamingResponse,
    AsyncSchedulesResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .destinations import (
    DestinationsResource,
    AsyncDestinationsResource,
    DestinationsResourceWithRawResponse,
    AsyncDestinationsResourceWithRawResponse,
    DestinationsResourceWithStreamingResponse,
    AsyncDestinationsResourceWithStreamingResponse,
)
from ..._base_client import make_request_options
from ...types.ad_hoc_response import AdHocResponse

__all__ = ["DataExportsResource", "AsyncDataExportsResource"]


class DataExportsResource(SyncAPIResource):
    @cached_property
    def destinations(self) -> DestinationsResource:
        return DestinationsResource(self._client)

    @cached_property
    def jobs(self) -> JobsResource:
        return JobsResource(self._client)

    @cached_property
    def schedules(self) -> SchedulesResource:
        return SchedulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> DataExportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return DataExportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DataExportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return DataExportsResourceWithStreamingResponse(self)

    @overload
    def create_adhoc(
        self,
        *,
        org_id: str | None = None,
        operational_data_types: List[
            Literal[
                "BILLS",
                "COMMITMENTS",
                "ACCOUNTS",
                "BALANCES",
                "CONTRACTS",
                "ACCOUNT_PLANS",
                "AGGREGATIONS",
                "PLANS",
                "PRICING",
                "PRICING_BANDS",
                "BILL_LINE_ITEMS",
                "METERS",
                "PRODUCTS",
                "COMPOUND_AGGREGATIONS",
                "PLAN_GROUPS",
                "PLAN_GROUP_LINKS",
                "PLAN_TEMPLATES",
                "BALANCE_TRANSACTIONS",
            ]
        ],
        source_type: Literal["USAGE", "OPERATIONAL"],
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AdHocResponse:
        """Trigger an ad-hoc Data Export.

        Each ad-hoc Export can be configured for
        exporting _only one of_ either Usage or Operational data:

        **Operational Data Exports**.

        - **Entity Types**. Use the `operationalDataTypes` parameter to specify the
          entities whose operational data you want to include in the ad-hoc export.
        - **Export Files**. For each of the entity types you select, when the ad-hoc
          export runs a separate file is compiled containing the operational data for
          all entities of that type that exist in your Organization

        **Usage Data Exports**.

        - **Meters/Accounts**. Select the Meters and Accounts whose usage data you want
          to include in the ad-hoc export.
        - **Aggregated or non-aggregated data**:

        1. If you _don't want to aggregate_ the usage data collected by the selected
           Meters, use **ORIGINAL** for `aggregationFrequency`, which is the _default_.
           This means the raw usage data collected by any type of Data Fields and the
           values for any Derived Fields on the selected Meters will be included in the
           ad-hoc export.
        2. If you _do want to aggregate_ the usage data collected by the selected
           Meters, use one of the other options for `aggregationFrequency`: **HOUR**,
           **DAY**, **WEEK**, or **MONTH**. You _must_ then also specified an
           `aggregation` method to be used on the usage data before export. Importantly,
           if you do aggregate usage data, only the usage data collected by any numeric
           Data Fields on the selected Meters - those of type **MEASURE**, **INCOME**,
           or **COST** - will be included in the ad-hoc export.

        **Date Range for Operational Data Exports**. To restrict the operational data
        included in the export by a date/time range, use the `startDate` and `endDate`
        date/time request parameters to specify the period. Constraints:

        - `startDate` must be before `endDate`.
        - `startDate` with no `endDate` is valid.
        - No `startDate` with `endDate` is valid.
        - `endDate` must be before present date/time.
        - Both are optional and if neither is defined, the export includes all data for
          selected entities.

        **Date Range for Usage Data Exports**. To restrict the usage data included in
        the export by date/time range, use the `timePeriod` request parameter to define
        a set date range. Alternatively, define a custom date range using the
        `startDate` and `endDate` date/time parameters:

        - Both `startDate` and `endDate` are required.
        - You cannot use a `startDate` earlier than 35 days in the past.
        - The `endDate` is valid up to tomorrow at 00:00.
        - You must define a Date Range using **timePeriod** or **startDate/endDate**,
          but they are mutually exclusive and you cannot use them together.

        **NOTE:** You can use the ExportJob `id` returned to check the status of the
        triggered ad-hoc export. See the
        [ExportJob](https://www.m3ter.com/docs/api#tag/ExportJob) section of this API
        Reference.

        Args:
          operational_data_types: The list of the operational data types should be exported for.

          source_type

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create_adhoc(
        self,
        *,
        org_id: str | None = None,
        aggregation_frequency: Literal["ORIGINAL", "HOUR", "DAY", "WEEK", "MONTH"],
        source_type: Literal["USAGE", "OPERATIONAL"],
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        aggregation: Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN"] | NotGiven = NOT_GIVEN,
        meter_ids: List[str] | NotGiven = NOT_GIVEN,
        time_period: Literal[
            "TODAY",
            "YESTERDAY",
            "WEEK_TO_DATE",
            "CURRENT_MONTH",
            "LAST_30_DAYS",
            "LAST_35_DAYS",
            "PREVIOUS_WEEK",
            "PREVIOUS_MONTH",
        ]
        | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AdHocResponse:
        """Trigger an ad-hoc Data Export.

        Each ad-hoc Export can be configured for
        exporting _only one of_ either Usage or Operational data:

        **Operational Data Exports**.

        - **Entity Types**. Use the `operationalDataTypes` parameter to specify the
          entities whose operational data you want to include in the ad-hoc export.
        - **Export Files**. For each of the entity types you select, when the ad-hoc
          export runs a separate file is compiled containing the operational data for
          all entities of that type that exist in your Organization

        **Usage Data Exports**.

        - **Meters/Accounts**. Select the Meters and Accounts whose usage data you want
          to include in the ad-hoc export.
        - **Aggregated or non-aggregated data**:

        1. If you _don't want to aggregate_ the usage data collected by the selected
           Meters, use **ORIGINAL** for `aggregationFrequency`, which is the _default_.
           This means the raw usage data collected by any type of Data Fields and the
           values for any Derived Fields on the selected Meters will be included in the
           ad-hoc export.
        2. If you _do want to aggregate_ the usage data collected by the selected
           Meters, use one of the other options for `aggregationFrequency`: **HOUR**,
           **DAY**, **WEEK**, or **MONTH**. You _must_ then also specified an
           `aggregation` method to be used on the usage data before export. Importantly,
           if you do aggregate usage data, only the usage data collected by any numeric
           Data Fields on the selected Meters - those of type **MEASURE**, **INCOME**,
           or **COST** - will be included in the ad-hoc export.

        **Date Range for Operational Data Exports**. To restrict the operational data
        included in the export by a date/time range, use the `startDate` and `endDate`
        date/time request parameters to specify the period. Constraints:

        - `startDate` must be before `endDate`.
        - `startDate` with no `endDate` is valid.
        - No `startDate` with `endDate` is valid.
        - `endDate` must be before present date/time.
        - Both are optional and if neither is defined, the export includes all data for
          selected entities.

        **Date Range for Usage Data Exports**. To restrict the usage data included in
        the export by date/time range, use the `timePeriod` request parameter to define
        a set date range. Alternatively, define a custom date range using the
        `startDate` and `endDate` date/time parameters:

        - Both `startDate` and `endDate` are required.
        - You cannot use a `startDate` earlier than 35 days in the past.
        - The `endDate` is valid up to tomorrow at 00:00.
        - You must define a Date Range using **timePeriod** or **startDate/endDate**,
          but they are mutually exclusive and you cannot use them together.

        **NOTE:** You can use the ExportJob `id` returned to check the status of the
        triggered ad-hoc export. See the
        [ExportJob](https://www.m3ter.com/docs/api#tag/ExportJob) section of this API
        Reference.

        Args:
          aggregation_frequency: Specifies the time period for the aggregation of usage data included each time
              the Data Export Schedule runs:

              - **ORIGINAL**. Usage data is _not aggregated_. If you select to not aggregate,
                then raw usage data measurements collected by all Data Field types and any
                Derived Fields on the selected Meters are included in the export. This is the
                _Default_.

              If you want to aggregate usage data for the Export Schedule you must define an
              `aggregationFrequency`:

              - **HOUR**. Aggregated hourly.
              - **DAY**. Aggregated daily.
              - **WEEK**. Aggregated weekly.
              - **MONTH**. Aggregated monthly.

              - If you select to aggregate usage data for a Export Schedule, then only the
                aggregated usage data collected by numeric Data Fields of type **MEASURE**,
                **INCOME**, or **COST** on selected Meters are included in the export.

              **NOTE**: If you define an `aggregationFrequency` other than **ORIGINAL** and do
              not define an `aggregation` method, then you'll receive and error.

          source_type

          account_ids: List of account IDs for which the usage data will be exported.

          aggregation: Specifies the aggregation method applied to usage data collected in the numeric
              Data Fields of Meters included for the Data Export Schedule - that is, Data
              Fields of type **MEASURE**, **INCOME**, or **COST**:

              - **SUM**. Adds the values.
              - **MIN**. Uses the minimum value.
              - **MAX**. Uses the maximum value.
              - **COUNT**. Counts the number of values.
              - **LATEST**. Uses the most recent value. Note: Based on the timestamp `ts`
                value of usage data measurement submissions. If using this method, please
                ensure _distinct_ `ts` values are used for usage data measurement submissions.

          meter_ids: List of meter IDs for which the usage data will be exported.

          time_period: Define a time period to control the range of usage data you want the data export
              to contain when it runs:

              - **TODAY**. Data collected for the current day up until the time the export
                runs.
              - **YESTERDAY**. Data collected for the day before the export runs - that is,
                the 24 hour period from midnight to midnight of the day before.
              - **WEEK_TO_DATE**. Data collected for the period covering the current week to
                the date and time the export runs, and weeks run Monday to Monday.
              - **CURRENT_MONTH**. Data collected for the current month in which the export is
                ran up to and including the date and time the export runs.
              - **LAST_30_DAYS**. Data collected for the 30 days prior to the date the export
                is ran.
              - **LAST_35_DAYS**. Data collected for the 35 days prior to the date the export
                is ran.
              - **PREVIOUS_WEEK**. Data collected for the previous full week period, and weeks
                run Monday to Monday.
              - **PREVIOUS_MONTH**. Data collected for the previous full month period.

              For more details and examples, see the
              [Time Period](https://www.m3ter.com/docs/guides/data-exports/creating-export-schedules#time-period)
              section in our main User Documentation.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["operational_data_types", "source_type"], ["aggregation_frequency", "source_type"])
    def create_adhoc(
        self,
        *,
        org_id: str | None = None,
        operational_data_types: List[
            Literal[
                "BILLS",
                "COMMITMENTS",
                "ACCOUNTS",
                "BALANCES",
                "CONTRACTS",
                "ACCOUNT_PLANS",
                "AGGREGATIONS",
                "PLANS",
                "PRICING",
                "PRICING_BANDS",
                "BILL_LINE_ITEMS",
                "METERS",
                "PRODUCTS",
                "COMPOUND_AGGREGATIONS",
                "PLAN_GROUPS",
                "PLAN_GROUP_LINKS",
                "PLAN_TEMPLATES",
                "BALANCE_TRANSACTIONS",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        source_type: Literal["USAGE", "OPERATIONAL"],
        version: int | NotGiven = NOT_GIVEN,
        aggregation_frequency: Literal["ORIGINAL", "HOUR", "DAY", "WEEK", "MONTH"] | NotGiven = NOT_GIVEN,
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        aggregation: Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN"] | NotGiven = NOT_GIVEN,
        meter_ids: List[str] | NotGiven = NOT_GIVEN,
        time_period: Literal[
            "TODAY",
            "YESTERDAY",
            "WEEK_TO_DATE",
            "CURRENT_MONTH",
            "LAST_30_DAYS",
            "LAST_35_DAYS",
            "PREVIOUS_WEEK",
            "PREVIOUS_MONTH",
        ]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AdHocResponse:
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/dataexports/adhoc",
            body=maybe_transform(
                {
                    "operational_data_types": operational_data_types,
                    "source_type": source_type,
                    "version": version,
                    "aggregation_frequency": aggregation_frequency,
                    "account_ids": account_ids,
                    "aggregation": aggregation,
                    "meter_ids": meter_ids,
                    "time_period": time_period,
                },
                data_export_create_adhoc_params.DataExportCreateAdhocParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AdHocResponse,
        )


class AsyncDataExportsResource(AsyncAPIResource):
    @cached_property
    def destinations(self) -> AsyncDestinationsResource:
        return AsyncDestinationsResource(self._client)

    @cached_property
    def jobs(self) -> AsyncJobsResource:
        return AsyncJobsResource(self._client)

    @cached_property
    def schedules(self) -> AsyncSchedulesResource:
        return AsyncSchedulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDataExportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDataExportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDataExportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncDataExportsResourceWithStreamingResponse(self)

    @overload
    async def create_adhoc(
        self,
        *,
        org_id: str | None = None,
        operational_data_types: List[
            Literal[
                "BILLS",
                "COMMITMENTS",
                "ACCOUNTS",
                "BALANCES",
                "CONTRACTS",
                "ACCOUNT_PLANS",
                "AGGREGATIONS",
                "PLANS",
                "PRICING",
                "PRICING_BANDS",
                "BILL_LINE_ITEMS",
                "METERS",
                "PRODUCTS",
                "COMPOUND_AGGREGATIONS",
                "PLAN_GROUPS",
                "PLAN_GROUP_LINKS",
                "PLAN_TEMPLATES",
                "BALANCE_TRANSACTIONS",
            ]
        ],
        source_type: Literal["USAGE", "OPERATIONAL"],
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AdHocResponse:
        """Trigger an ad-hoc Data Export.

        Each ad-hoc Export can be configured for
        exporting _only one of_ either Usage or Operational data:

        **Operational Data Exports**.

        - **Entity Types**. Use the `operationalDataTypes` parameter to specify the
          entities whose operational data you want to include in the ad-hoc export.
        - **Export Files**. For each of the entity types you select, when the ad-hoc
          export runs a separate file is compiled containing the operational data for
          all entities of that type that exist in your Organization

        **Usage Data Exports**.

        - **Meters/Accounts**. Select the Meters and Accounts whose usage data you want
          to include in the ad-hoc export.
        - **Aggregated or non-aggregated data**:

        1. If you _don't want to aggregate_ the usage data collected by the selected
           Meters, use **ORIGINAL** for `aggregationFrequency`, which is the _default_.
           This means the raw usage data collected by any type of Data Fields and the
           values for any Derived Fields on the selected Meters will be included in the
           ad-hoc export.
        2. If you _do want to aggregate_ the usage data collected by the selected
           Meters, use one of the other options for `aggregationFrequency`: **HOUR**,
           **DAY**, **WEEK**, or **MONTH**. You _must_ then also specified an
           `aggregation` method to be used on the usage data before export. Importantly,
           if you do aggregate usage data, only the usage data collected by any numeric
           Data Fields on the selected Meters - those of type **MEASURE**, **INCOME**,
           or **COST** - will be included in the ad-hoc export.

        **Date Range for Operational Data Exports**. To restrict the operational data
        included in the export by a date/time range, use the `startDate` and `endDate`
        date/time request parameters to specify the period. Constraints:

        - `startDate` must be before `endDate`.
        - `startDate` with no `endDate` is valid.
        - No `startDate` with `endDate` is valid.
        - `endDate` must be before present date/time.
        - Both are optional and if neither is defined, the export includes all data for
          selected entities.

        **Date Range for Usage Data Exports**. To restrict the usage data included in
        the export by date/time range, use the `timePeriod` request parameter to define
        a set date range. Alternatively, define a custom date range using the
        `startDate` and `endDate` date/time parameters:

        - Both `startDate` and `endDate` are required.
        - You cannot use a `startDate` earlier than 35 days in the past.
        - The `endDate` is valid up to tomorrow at 00:00.
        - You must define a Date Range using **timePeriod** or **startDate/endDate**,
          but they are mutually exclusive and you cannot use them together.

        **NOTE:** You can use the ExportJob `id` returned to check the status of the
        triggered ad-hoc export. See the
        [ExportJob](https://www.m3ter.com/docs/api#tag/ExportJob) section of this API
        Reference.

        Args:
          operational_data_types: The list of the operational data types should be exported for.

          source_type

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create_adhoc(
        self,
        *,
        org_id: str | None = None,
        aggregation_frequency: Literal["ORIGINAL", "HOUR", "DAY", "WEEK", "MONTH"],
        source_type: Literal["USAGE", "OPERATIONAL"],
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        aggregation: Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN"] | NotGiven = NOT_GIVEN,
        meter_ids: List[str] | NotGiven = NOT_GIVEN,
        time_period: Literal[
            "TODAY",
            "YESTERDAY",
            "WEEK_TO_DATE",
            "CURRENT_MONTH",
            "LAST_30_DAYS",
            "LAST_35_DAYS",
            "PREVIOUS_WEEK",
            "PREVIOUS_MONTH",
        ]
        | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AdHocResponse:
        """Trigger an ad-hoc Data Export.

        Each ad-hoc Export can be configured for
        exporting _only one of_ either Usage or Operational data:

        **Operational Data Exports**.

        - **Entity Types**. Use the `operationalDataTypes` parameter to specify the
          entities whose operational data you want to include in the ad-hoc export.
        - **Export Files**. For each of the entity types you select, when the ad-hoc
          export runs a separate file is compiled containing the operational data for
          all entities of that type that exist in your Organization

        **Usage Data Exports**.

        - **Meters/Accounts**. Select the Meters and Accounts whose usage data you want
          to include in the ad-hoc export.
        - **Aggregated or non-aggregated data**:

        1. If you _don't want to aggregate_ the usage data collected by the selected
           Meters, use **ORIGINAL** for `aggregationFrequency`, which is the _default_.
           This means the raw usage data collected by any type of Data Fields and the
           values for any Derived Fields on the selected Meters will be included in the
           ad-hoc export.
        2. If you _do want to aggregate_ the usage data collected by the selected
           Meters, use one of the other options for `aggregationFrequency`: **HOUR**,
           **DAY**, **WEEK**, or **MONTH**. You _must_ then also specified an
           `aggregation` method to be used on the usage data before export. Importantly,
           if you do aggregate usage data, only the usage data collected by any numeric
           Data Fields on the selected Meters - those of type **MEASURE**, **INCOME**,
           or **COST** - will be included in the ad-hoc export.

        **Date Range for Operational Data Exports**. To restrict the operational data
        included in the export by a date/time range, use the `startDate` and `endDate`
        date/time request parameters to specify the period. Constraints:

        - `startDate` must be before `endDate`.
        - `startDate` with no `endDate` is valid.
        - No `startDate` with `endDate` is valid.
        - `endDate` must be before present date/time.
        - Both are optional and if neither is defined, the export includes all data for
          selected entities.

        **Date Range for Usage Data Exports**. To restrict the usage data included in
        the export by date/time range, use the `timePeriod` request parameter to define
        a set date range. Alternatively, define a custom date range using the
        `startDate` and `endDate` date/time parameters:

        - Both `startDate` and `endDate` are required.
        - You cannot use a `startDate` earlier than 35 days in the past.
        - The `endDate` is valid up to tomorrow at 00:00.
        - You must define a Date Range using **timePeriod** or **startDate/endDate**,
          but they are mutually exclusive and you cannot use them together.

        **NOTE:** You can use the ExportJob `id` returned to check the status of the
        triggered ad-hoc export. See the
        [ExportJob](https://www.m3ter.com/docs/api#tag/ExportJob) section of this API
        Reference.

        Args:
          aggregation_frequency: Specifies the time period for the aggregation of usage data included each time
              the Data Export Schedule runs:

              - **ORIGINAL**. Usage data is _not aggregated_. If you select to not aggregate,
                then raw usage data measurements collected by all Data Field types and any
                Derived Fields on the selected Meters are included in the export. This is the
                _Default_.

              If you want to aggregate usage data for the Export Schedule you must define an
              `aggregationFrequency`:

              - **HOUR**. Aggregated hourly.
              - **DAY**. Aggregated daily.
              - **WEEK**. Aggregated weekly.
              - **MONTH**. Aggregated monthly.

              - If you select to aggregate usage data for a Export Schedule, then only the
                aggregated usage data collected by numeric Data Fields of type **MEASURE**,
                **INCOME**, or **COST** on selected Meters are included in the export.

              **NOTE**: If you define an `aggregationFrequency` other than **ORIGINAL** and do
              not define an `aggregation` method, then you'll receive and error.

          source_type

          account_ids: List of account IDs for which the usage data will be exported.

          aggregation: Specifies the aggregation method applied to usage data collected in the numeric
              Data Fields of Meters included for the Data Export Schedule - that is, Data
              Fields of type **MEASURE**, **INCOME**, or **COST**:

              - **SUM**. Adds the values.
              - **MIN**. Uses the minimum value.
              - **MAX**. Uses the maximum value.
              - **COUNT**. Counts the number of values.
              - **LATEST**. Uses the most recent value. Note: Based on the timestamp `ts`
                value of usage data measurement submissions. If using this method, please
                ensure _distinct_ `ts` values are used for usage data measurement submissions.

          meter_ids: List of meter IDs for which the usage data will be exported.

          time_period: Define a time period to control the range of usage data you want the data export
              to contain when it runs:

              - **TODAY**. Data collected for the current day up until the time the export
                runs.
              - **YESTERDAY**. Data collected for the day before the export runs - that is,
                the 24 hour period from midnight to midnight of the day before.
              - **WEEK_TO_DATE**. Data collected for the period covering the current week to
                the date and time the export runs, and weeks run Monday to Monday.
              - **CURRENT_MONTH**. Data collected for the current month in which the export is
                ran up to and including the date and time the export runs.
              - **LAST_30_DAYS**. Data collected for the 30 days prior to the date the export
                is ran.
              - **LAST_35_DAYS**. Data collected for the 35 days prior to the date the export
                is ran.
              - **PREVIOUS_WEEK**. Data collected for the previous full week period, and weeks
                run Monday to Monday.
              - **PREVIOUS_MONTH**. Data collected for the previous full month period.

              For more details and examples, see the
              [Time Period](https://www.m3ter.com/docs/guides/data-exports/creating-export-schedules#time-period)
              section in our main User Documentation.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["operational_data_types", "source_type"], ["aggregation_frequency", "source_type"])
    async def create_adhoc(
        self,
        *,
        org_id: str | None = None,
        operational_data_types: List[
            Literal[
                "BILLS",
                "COMMITMENTS",
                "ACCOUNTS",
                "BALANCES",
                "CONTRACTS",
                "ACCOUNT_PLANS",
                "AGGREGATIONS",
                "PLANS",
                "PRICING",
                "PRICING_BANDS",
                "BILL_LINE_ITEMS",
                "METERS",
                "PRODUCTS",
                "COMPOUND_AGGREGATIONS",
                "PLAN_GROUPS",
                "PLAN_GROUP_LINKS",
                "PLAN_TEMPLATES",
                "BALANCE_TRANSACTIONS",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        source_type: Literal["USAGE", "OPERATIONAL"],
        version: int | NotGiven = NOT_GIVEN,
        aggregation_frequency: Literal["ORIGINAL", "HOUR", "DAY", "WEEK", "MONTH"] | NotGiven = NOT_GIVEN,
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        aggregation: Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN"] | NotGiven = NOT_GIVEN,
        meter_ids: List[str] | NotGiven = NOT_GIVEN,
        time_period: Literal[
            "TODAY",
            "YESTERDAY",
            "WEEK_TO_DATE",
            "CURRENT_MONTH",
            "LAST_30_DAYS",
            "LAST_35_DAYS",
            "PREVIOUS_WEEK",
            "PREVIOUS_MONTH",
        ]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AdHocResponse:
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/dataexports/adhoc",
            body=await async_maybe_transform(
                {
                    "operational_data_types": operational_data_types,
                    "source_type": source_type,
                    "version": version,
                    "aggregation_frequency": aggregation_frequency,
                    "account_ids": account_ids,
                    "aggregation": aggregation,
                    "meter_ids": meter_ids,
                    "time_period": time_period,
                },
                data_export_create_adhoc_params.DataExportCreateAdhocParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AdHocResponse,
        )


class DataExportsResourceWithRawResponse:
    def __init__(self, data_exports: DataExportsResource) -> None:
        self._data_exports = data_exports

        self.create_adhoc = to_raw_response_wrapper(
            data_exports.create_adhoc,
        )

    @cached_property
    def destinations(self) -> DestinationsResourceWithRawResponse:
        return DestinationsResourceWithRawResponse(self._data_exports.destinations)

    @cached_property
    def jobs(self) -> JobsResourceWithRawResponse:
        return JobsResourceWithRawResponse(self._data_exports.jobs)

    @cached_property
    def schedules(self) -> SchedulesResourceWithRawResponse:
        return SchedulesResourceWithRawResponse(self._data_exports.schedules)


class AsyncDataExportsResourceWithRawResponse:
    def __init__(self, data_exports: AsyncDataExportsResource) -> None:
        self._data_exports = data_exports

        self.create_adhoc = async_to_raw_response_wrapper(
            data_exports.create_adhoc,
        )

    @cached_property
    def destinations(self) -> AsyncDestinationsResourceWithRawResponse:
        return AsyncDestinationsResourceWithRawResponse(self._data_exports.destinations)

    @cached_property
    def jobs(self) -> AsyncJobsResourceWithRawResponse:
        return AsyncJobsResourceWithRawResponse(self._data_exports.jobs)

    @cached_property
    def schedules(self) -> AsyncSchedulesResourceWithRawResponse:
        return AsyncSchedulesResourceWithRawResponse(self._data_exports.schedules)


class DataExportsResourceWithStreamingResponse:
    def __init__(self, data_exports: DataExportsResource) -> None:
        self._data_exports = data_exports

        self.create_adhoc = to_streamed_response_wrapper(
            data_exports.create_adhoc,
        )

    @cached_property
    def destinations(self) -> DestinationsResourceWithStreamingResponse:
        return DestinationsResourceWithStreamingResponse(self._data_exports.destinations)

    @cached_property
    def jobs(self) -> JobsResourceWithStreamingResponse:
        return JobsResourceWithStreamingResponse(self._data_exports.jobs)

    @cached_property
    def schedules(self) -> SchedulesResourceWithStreamingResponse:
        return SchedulesResourceWithStreamingResponse(self._data_exports.schedules)


class AsyncDataExportsResourceWithStreamingResponse:
    def __init__(self, data_exports: AsyncDataExportsResource) -> None:
        self._data_exports = data_exports

        self.create_adhoc = async_to_streamed_response_wrapper(
            data_exports.create_adhoc,
        )

    @cached_property
    def destinations(self) -> AsyncDestinationsResourceWithStreamingResponse:
        return AsyncDestinationsResourceWithStreamingResponse(self._data_exports.destinations)

    @cached_property
    def jobs(self) -> AsyncJobsResourceWithStreamingResponse:
        return AsyncJobsResourceWithStreamingResponse(self._data_exports.jobs)

    @cached_property
    def schedules(self) -> AsyncSchedulesResourceWithStreamingResponse:
        return AsyncSchedulesResourceWithStreamingResponse(self._data_exports.schedules)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ExternalMappingUpdateParams"]


class ExternalMappingUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    external_id: Required[Annotated[str, PropertyInfo(alias="externalId")]]
    """The unique identifier (UUID) of the entity in the external system.

    This UUID should already exist in the external system.
    """

    external_system: Required[Annotated[str, PropertyInfo(alias="externalSystem")]]
    """The name of the external system where the entity you are mapping resides."""

    external_table: Required[Annotated[str, PropertyInfo(alias="externalTable")]]
    """The name of the table in ther external system where the entity resides."""

    m3ter_entity: Required[Annotated[str, PropertyInfo(alias="m3terEntity")]]
    """
    The name of the m3ter entity that you are creating or modifying an External
    Mapping for. As an example, this could be an "Account".
    """

    m3ter_id: Required[Annotated[str, PropertyInfo(alias="m3terId")]]
    """The unique identifier (UUID) of the m3ter entity."""

    integration_config_id: Annotated[str, PropertyInfo(alias="integrationConfigId")]
    """UUID of the integration config to link this mapping to"""

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ResourceGroupCreateParams"]


class ResourceGroupCreateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    name: Required[str]

    version: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["BillUpdateStatusParams"]


class BillUpdateStatusParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    status: Required[Literal["PENDING", "APPROVED"]]
    """The new status you want to assign to the Bill.

    Must be one "Pending" or "Approved".
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["M3terSignedCredentialsRequestParam"]


class M3terSignedCredentialsRequestParam(TypedDict, total=False):
    api_key: Required[Annotated[str, PropertyInfo(alias="apiKey")]]
    """The API key provided by m3ter.

    This key is part of the credential set required for signing requests and
    authenticating with m3ter services.
    """

    secret: Required[str]
    """The secret associated with the API key.

    This secret is used in conjunction with the API key to generate a signature for
    secure authentication.
    """

    type: Required[Literal["M3TER_SIGNED_REQUEST"]]
    """Specifies the authorization type.

    For this schema, it is exclusively set to M3TER_SIGNED_REQUEST.
    """

    empty: bool
    """A flag to indicate whether the credentials are empty.

    - TRUE - empty credentials.
    - FALSE - credential details required.
    """

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .line_item_response import LineItemResponse as LineItemResponse
from .line_item_list_params import LineItemListParams as LineItemListParams
from .debit_line_item_response import DebitLineItemResponse as DebitLineItemResponse
from .credit_line_item_response import CreditLineItemResponse as CreditLineItemResponse
from .debit_line_item_list_params import DebitLineItemListParams as DebitLineItemListParams
from .credit_line_item_list_params import CreditLineItemListParams as CreditLineItemListParams
from .debit_line_item_create_params import DebitLineItemCreateParams as DebitLineItemCreateParams
from .debit_line_item_update_params import DebitLineItemUpdateParams as DebitLineItemUpdateParams
from .credit_line_item_create_params import CreditLineItemCreateParams as CreditLineItemCreateParams
from .credit_line_item_update_params import CreditLineItemUpdateParams as CreditLineItemUpdateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["CommitmentFee"]


class CommitmentFee(BaseModel):
    amount: float

    date: datetime.date

    service_period_end_date: datetime.datetime = FieldInfo(alias="servicePeriodEndDate")

    service_period_start_date: datetime.datetime = FieldInfo(alias="servicePeriodStartDate")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["CreditLineItemUpdateParams"]


class CreditLineItemUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    bill_id: Required[Annotated[str, PropertyInfo(alias="billId")]]

    amount: Required[float]
    """The amount for the line item."""

    description: Required[str]
    """The description for the line item."""

    product_id: Required[Annotated[str, PropertyInfo(alias="productId")]]
    """The UUID of the Product."""

    referenced_bill_id: Required[Annotated[str, PropertyInfo(alias="referencedBillId")]]
    """The UUID of the bill for the line item."""

    referenced_line_item_id: Required[Annotated[str, PropertyInfo(alias="referencedLineItemId")]]
    """The UUID of the line item."""

    service_period_end_date: Required[
        Annotated[Union[str, datetime], PropertyInfo(alias="servicePeriodEndDate", format="iso8601")]
    ]
    """
    The service period end date in ISO-8601 format._(exclusive of the ending date)_.
    """

    service_period_start_date: Required[
        Annotated[Union[str, datetime], PropertyInfo(alias="servicePeriodStartDate", format="iso8601")]
    ]
    """The service period start date in ISO-8601 format.

    _(inclusive of the starting date)_.
    """

    credit_reason_id: Annotated[str, PropertyInfo(alias="creditReasonId")]
    """The UUID of the credit reason."""

    line_item_type: Annotated[
        Literal[
            "STANDING_CHARGE",
            "USAGE",
            "COUNTER_RUNNING_TOTAL_CHARGE",
            "COUNTER_ADJUSTMENT_DEBIT",
            "COUNTER_ADJUSTMENT_CREDIT",
            "USAGE_CREDIT",
            "MINIMUM_SPEND",
            "MINIMUM_SPEND_REFUND",
            "CREDIT_DEDUCTION",
            "MANUAL_ADJUSTMENT",
            "CREDIT_MEMO",
            "DEBIT_MEMO",
            "COMMITMENT_CONSUMED",
            "COMMITMENT_FEE",
            "OVERAGE_SURCHARGE",
            "OVERAGE_USAGE",
            "BALANCE_CONSUMED",
            "BALANCE_FEE",
        ],
        PropertyInfo(alias="lineItemType"),
    ]

    reason_id: Annotated[str, PropertyInfo(alias="reasonId")]
    """The UUID of the line item reason."""

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from m3ter import M3ter, AsyncM3ter
from m3ter.types import BillJobResponse
from tests.utils import assert_matches_type
from m3ter._utils import parse_date
from m3ter.pagination import SyncCursor, AsyncCursor

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBillJobs:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: M3ter) -> None:
        bill_job = client.bill_jobs.create(
            org_id="orgId",
        )
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: M3ter) -> None:
        bill_job = client.bill_jobs.create(
            org_id="orgId",
            account_ids=["string"],
            bill_date=parse_date("2019-12-27"),
            bill_frequency_interval=0,
            billing_frequency="DAILY",
            currency_conversions=[
                {
                    "from": "EUR",
                    "to": "USD",
                    "multiplier": 1.12,
                }
            ],
            day_epoch=parse_date("2019-12-27"),
            due_date=parse_date("2019-12-27"),
            external_invoice_date=parse_date("2019-12-27"),
            last_date_in_billing_period=parse_date("2019-12-27"),
            month_epoch=parse_date("2019-12-27"),
            target_currency="xxx",
            timezone="UTC",
            version=0,
            week_epoch=parse_date("2019-12-27"),
            year_epoch=parse_date("2019-12-27"),
        )
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: M3ter) -> None:
        response = client.bill_jobs.with_raw_response.create(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill_job = response.parse()
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: M3ter) -> None:
        with client.bill_jobs.with_streaming_response.create(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill_job = response.parse()
            assert_matches_type(BillJobResponse, bill_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.bill_jobs.with_raw_response.create(
                org_id="",
            )

    @parametrize
    def test_method_retrieve(self, client: M3ter) -> None:
        bill_job = client.bill_jobs.retrieve(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: M3ter) -> None:
        response = client.bill_jobs.with_raw_response.retrieve(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill_job = response.parse()
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: M3ter) -> None:
        with client.bill_jobs.with_streaming_response.retrieve(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill_job = response.parse()
            assert_matches_type(BillJobResponse, bill_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.bill_jobs.with_raw_response.retrieve(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.bill_jobs.with_raw_response.retrieve(
                id="",
                org_id="orgId",
            )

    @parametrize
    def test_method_list(self, client: M3ter) -> None:
        bill_job = client.bill_jobs.list(
            org_id="orgId",
        )
        assert_matches_type(SyncCursor[BillJobResponse], bill_job, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: M3ter) -> None:
        bill_job = client.bill_jobs.list(
            org_id="orgId",
            active="active",
            next_token="nextToken",
            page_size=1,
            status="status",
        )
        assert_matches_type(SyncCursor[BillJobResponse], bill_job, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: M3ter) -> None:
        response = client.bill_jobs.with_raw_response.list(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill_job = response.parse()
        assert_matches_type(SyncCursor[BillJobResponse], bill_job, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: M3ter) -> None:
        with client.bill_jobs.with_streaming_response.list(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill_job = response.parse()
            assert_matches_type(SyncCursor[BillJobResponse], bill_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.bill_jobs.with_raw_response.list(
                org_id="",
            )

    @parametrize
    def test_method_cancel(self, client: M3ter) -> None:
        bill_job = client.bill_jobs.cancel(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    def test_raw_response_cancel(self, client: M3ter) -> None:
        response = client.bill_jobs.with_raw_response.cancel(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill_job = response.parse()
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    def test_streaming_response_cancel(self, client: M3ter) -> None:
        with client.bill_jobs.with_streaming_response.cancel(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill_job = response.parse()
            assert_matches_type(BillJobResponse, bill_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_cancel(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.bill_jobs.with_raw_response.cancel(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.bill_jobs.with_raw_response.cancel(
                id="",
                org_id="orgId",
            )

    @parametrize
    def test_method_recalculate(self, client: M3ter) -> None:
        bill_job = client.bill_jobs.recalculate(
            org_id="orgId",
            bill_ids=["string"],
        )
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    def test_method_recalculate_with_all_params(self, client: M3ter) -> None:
        bill_job = client.bill_jobs.recalculate(
            org_id="orgId",
            bill_ids=["string"],
            version=0,
        )
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    def test_raw_response_recalculate(self, client: M3ter) -> None:
        response = client.bill_jobs.with_raw_response.recalculate(
            org_id="orgId",
            bill_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill_job = response.parse()
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    def test_streaming_response_recalculate(self, client: M3ter) -> None:
        with client.bill_jobs.with_streaming_response.recalculate(
            org_id="orgId",
            bill_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill_job = response.parse()
            assert_matches_type(BillJobResponse, bill_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_recalculate(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.bill_jobs.with_raw_response.recalculate(
                org_id="",
                bill_ids=["string"],
            )


class TestAsyncBillJobs:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncM3ter) -> None:
        bill_job = await async_client.bill_jobs.create(
            org_id="orgId",
        )
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncM3ter) -> None:
        bill_job = await async_client.bill_jobs.create(
            org_id="orgId",
            account_ids=["string"],
            bill_date=parse_date("2019-12-27"),
            bill_frequency_interval=0,
            billing_frequency="DAILY",
            currency_conversions=[
                {
                    "from": "EUR",
                    "to": "USD",
                    "multiplier": 1.12,
                }
            ],
            day_epoch=parse_date("2019-12-27"),
            due_date=parse_date("2019-12-27"),
            external_invoice_date=parse_date("2019-12-27"),
            last_date_in_billing_period=parse_date("2019-12-27"),
            month_epoch=parse_date("2019-12-27"),
            target_currency="xxx",
            timezone="UTC",
            version=0,
            week_epoch=parse_date("2019-12-27"),
            year_epoch=parse_date("2019-12-27"),
        )
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncM3ter) -> None:
        response = await async_client.bill_jobs.with_raw_response.create(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill_job = await response.parse()
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncM3ter) -> None:
        async with async_client.bill_jobs.with_streaming_response.create(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill_job = await response.parse()
            assert_matches_type(BillJobResponse, bill_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.bill_jobs.with_raw_response.create(
                org_id="",
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncM3ter) -> None:
        bill_job = await async_client.bill_jobs.retrieve(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncM3ter) -> None:
        response = await async_client.bill_jobs.with_raw_response.retrieve(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill_job = await response.parse()
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncM3ter) -> None:
        async with async_client.bill_jobs.with_streaming_response.retrieve(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill_job = await response.parse()
            assert_matches_type(BillJobResponse, bill_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.bill_jobs.with_raw_response.retrieve(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.bill_jobs.with_raw_response.retrieve(
                id="",
                org_id="orgId",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncM3ter) -> None:
        bill_job = await async_client.bill_jobs.list(
            org_id="orgId",
        )
        assert_matches_type(AsyncCursor[BillJobResponse], bill_job, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncM3ter) -> None:
        bill_job = await async_client.bill_jobs.list(
            org_id="orgId",
            active="active",
            next_token="nextToken",
            page_size=1,
            status="status",
        )
        assert_matches_type(AsyncCursor[BillJobResponse], bill_job, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncM3ter) -> None:
        response = await async_client.bill_jobs.with_raw_response.list(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill_job = await response.parse()
        assert_matches_type(AsyncCursor[BillJobResponse], bill_job, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncM3ter) -> None:
        async with async_client.bill_jobs.with_streaming_response.list(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill_job = await response.parse()
            assert_matches_type(AsyncCursor[BillJobResponse], bill_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.bill_jobs.with_raw_response.list(
                org_id="",
            )

    @parametrize
    async def test_method_cancel(self, async_client: AsyncM3ter) -> None:
        bill_job = await async_client.bill_jobs.cancel(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    async def test_raw_response_cancel(self, async_client: AsyncM3ter) -> None:
        response = await async_client.bill_jobs.with_raw_response.cancel(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill_job = await response.parse()
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    async def test_streaming_response_cancel(self, async_client: AsyncM3ter) -> None:
        async with async_client.bill_jobs.with_streaming_response.cancel(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill_job = await response.parse()
            assert_matches_type(BillJobResponse, bill_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_cancel(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.bill_jobs.with_raw_response.cancel(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.bill_jobs.with_raw_response.cancel(
                id="",
                org_id="orgId",
            )

    @parametrize
    async def test_method_recalculate(self, async_client: AsyncM3ter) -> None:
        bill_job = await async_client.bill_jobs.recalculate(
            org_id="orgId",
            bill_ids=["string"],
        )
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    async def test_method_recalculate_with_all_params(self, async_client: AsyncM3ter) -> None:
        bill_job = await async_client.bill_jobs.recalculate(
            org_id="orgId",
            bill_ids=["string"],
            version=0,
        )
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    async def test_raw_response_recalculate(self, async_client: AsyncM3ter) -> None:
        response = await async_client.bill_jobs.with_raw_response.recalculate(
            org_id="orgId",
            bill_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bill_job = await response.parse()
        assert_matches_type(BillJobResponse, bill_job, path=["response"])

    @parametrize
    async def test_streaming_response_recalculate(self, async_client: AsyncM3ter) -> None:
        async with async_client.bill_jobs.with_streaming_response.recalculate(
            org_id="orgId",
            bill_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bill_job = await response.parse()
            assert_matches_type(BillJobResponse, bill_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_recalculate(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.bill_jobs.with_raw_response.recalculate(
                org_id="",
                bill_ids=["string"],
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import datetime
from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["CounterAdjustmentResponse"]


class CounterAdjustmentResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    account_id: Optional[str] = FieldInfo(alias="accountId", default=None)
    """The Account ID the CounterAdjustment was created for."""

    counter_id: Optional[str] = FieldInfo(alias="counterId", default=None)
    """
    The ID of the Counter that was used to make the CounterAdjustment on the
    Account.
    """

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The ID of the user who created this item."""

    date: Optional[datetime.date] = None
    """
    The date the CounterAdjustment was created for the Account _(in ISO-8601 date
    format)_.
    """

    dt_created: Optional[datetime.datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when this item was created _(in ISO-8601 format)_."""

    dt_last_modified: Optional[datetime.datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when this item was last modified _(in ISO-8601 format)_."""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The ID of the user who last modified this item."""

    purchase_order_number: Optional[str] = FieldInfo(alias="purchaseOrderNumber", default=None)
    """Purchase Order Number for the Counter Adjustment. _(Optional)_"""

    value: Optional[int] = None
    """Integer Value of the Counter that was used to make the CounterAdjustment."""
